@@ -9,16 +9,22 @@ import sys
 import time
 import traceback
 
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.asymmetric import ec, rsa
+from cryptography.hazmat.primitives.serialization import load_pem_private_key
 import OpenSSL
-import six
 import zope.component
 
 from acme.magic_typing import List
+from acme.magic_typing import Optional  # pylint: disable=unused-import
 from certbot import crypto_util
+from certbot.display import util as display_util
 from certbot import errors
 from certbot import interfaces
 from certbot import util
 from certbot._internal import cli
+from certbot._internal import client  # pylint: disable=unused-import
+from certbot._internal import constants
 from certbot._internal import hooks
 from certbot._internal import storage
 from certbot._internal import updater
@@ -33,7 +39,8 @@ logger = logging.getLogger(__name__)
 # the renewal configuration process loses this information.
 STR_CONFIG_ITEMS = ["config_dir", "logs_dir", "work_dir", "user_agent",
                     "server", "account", "authenticator", "installer",
-                    "renew_hook", "pre_hook", "post_hook", "http01_address"]
+                    "renew_hook", "pre_hook", "post_hook", "http01_address",
+                    "preferred_chain", "key_type", "elliptic_curve"]
 INT_CONFIG_ITEMS = ["rsa_key_size", "http01_port"]
 BOOL_CONFIG_ITEMS = ["must_staple", "allow_subset_of_names", "reuse_key",
                      "autorenew"]
@@ -77,6 +84,7 @@ def _reconstitute(config, full_path):
         return None
     # Now restore specific values along with their data types, if
     # those elements are present.
+    renewalparams = _remove_deprecated_config_elements(renewalparams)
     try:
         restore_required_config_elements(config, renewalparams)
         _restore_plugin_configs(config, renewalparams)
@@ -91,7 +99,7 @@ def _reconstitute(config, full_path):
         config.domains = [util.enforce_domain_sanity(d)
                           for d in renewal_candidate.names()]
     except errors.ConfigurationError as error:
-        logger.warning("Renewal configuration file %s references a cert "
+        logger.warning("Renewal configuration file %s references a certificate "
                        "that contains an invalid domain name. The problem "
                        "was: %s. Skipping.", full_path, error)
         return None
@@ -111,7 +119,7 @@ def _restore_webroot_config(config, renewalparams):
     # see https://github.com/certbot/certbot/pull/7095
     if "webroot_path" in renewalparams and not cli.set_by_cli("webroot_path"):
         wp = renewalparams["webroot_path"]
-        if isinstance(wp, six.string_types):  # prior to 0.1.0, webroot_path was a string
+        if isinstance(wp, str):  # prior to 0.1.0, webroot_path was a string
             wp = [wp]
         config.webroot_path = wp
 
@@ -146,7 +154,7 @@ def _restore_plugin_configs(config, renewalparams):
 
     for plugin_prefix in set(plugin_prefixes):
         plugin_prefix = plugin_prefix.replace('-', '_')
-        for config_item, config_value in six.iteritems(renewalparams):
+        for config_item, config_value in renewalparams.items():
             if config_item.startswith(plugin_prefix + "_") and not cli.set_by_cli(config_item):
                 # Values None, True, and False need to be treated specially,
                 # As their types aren't handled correctly by configobj
@@ -171,15 +179,28 @@ def restore_required_config_elements(config, renewalparams):
 
     required_items = itertools.chain(
         (("pref_challs", _restore_pref_challs),),
-        six.moves.zip(BOOL_CONFIG_ITEMS, itertools.repeat(_restore_bool)),
-        six.moves.zip(INT_CONFIG_ITEMS, itertools.repeat(_restore_int)),
-        six.moves.zip(STR_CONFIG_ITEMS, itertools.repeat(_restore_str)))
+        zip(BOOL_CONFIG_ITEMS, itertools.repeat(_restore_bool)),
+        zip(INT_CONFIG_ITEMS, itertools.repeat(_restore_int)),
+        zip(STR_CONFIG_ITEMS, itertools.repeat(_restore_str)))
     for item_name, restore_func in required_items:
         if item_name in renewalparams and not cli.set_by_cli(item_name):
             value = restore_func(item_name, renewalparams[item_name])
             setattr(config, item_name, value)
 
 
+def _remove_deprecated_config_elements(renewalparams):
+    """Removes deprecated config options from the parsed renewalparams.
+
+    :param dict renewalparams: list of parsed renewalparams
+
+    :returns: list of renewalparams with deprecated config options removed
+    :rtype: dict
+
+    """
+    return {option_name: v for (option_name, v) in renewalparams.items()
+        if option_name not in cli.DEPRECATED_OPTIONS}
+
+
 def _restore_pref_challs(unused_name, value):
     """Restores preferred challenges from a renewal config file.
 
@@ -198,7 +219,7 @@ def _restore_pref_challs(unused_name, value):
     # If pref_challs has only one element, configobj saves the value
     # with a trailing comma so it's parsed as a list. If this comma is
     # removed by the user, the value is parsed as a str.
-    value = [value] if isinstance(value, six.string_types) else value
+    value = [value] if isinstance(value, str) else value
     return cli.parse_preferred_challenges(value)
 
 
@@ -243,16 +264,28 @@ def _restore_int(name, value):
         raise errors.Error("Expected a numeric value for {0}".format(name))
 
 
-def _restore_str(unused_name, value):
+def _restore_str(name, value):
     """Restores a string key-value pair from a renewal config file.
 
-    :param str unused_name: option name
+    :param str name: option name
     :param str value: option value
 
     :returns: converted option value to be stored in the runtime config
     :rtype: str or None
 
     """
+    # Previous to v0.5.0, Certbot always stored the `server` URL in the renewal config,
+    # resulting in configs which explicitly use the deprecated ACMEv1 URL, today
+    # preventing an automatic transition to the default modern ACME URL.
+    # (https://github.com/certbot/certbot/issues/7978#issuecomment-625442870)
+    # As a mitigation, this function reinterprets the value of the `server` parameter if
+    # necessary, replacing the ACMEv1 URL with the default ACME URL. It is still possible
+    # to override this choice with the explicit `--server` CLI flag.
+    if name == "server" and value == constants.V1_URI:
+        logger.info("Using server %s instead of legacy %s",
+                    constants.CLI_DEFAULTS["server"], value)
+        return constants.CLI_DEFAULTS["server"]
+
     return None if value == "None" else value
 
 
@@ -273,18 +306,15 @@ def should_renew(config, lineage):
 
 def _avoid_invalidating_lineage(config, lineage, original_server):
     "Do not renew a valid cert with one from a staging server!"
-    # Some lineages may have begun with --staging, but then had production certs
-    # added to them
+    # Some lineages may have begun with --staging, but then had production
+    # certificates added to them
     with open(lineage.cert) as the_file:
         contents = the_file.read()
     latest_cert = OpenSSL.crypto.load_certificate(
         OpenSSL.crypto.FILETYPE_PEM, contents)
-    # all our test certs are from happy hacker fake CA, though maybe one day
-    # we should test more methodically
-    now_valid = "fake" not in repr(latest_cert.get_issuer()).lower()
 
     if util.is_staging(config.server):
-        if not util.is_staging(original_server) or now_valid:
+        if not util.is_staging(original_server):
             if not config.break_my_certs:
                 names = ", ".join(lineage.names())
                 raise errors.Error(
@@ -294,7 +324,8 @@ def _avoid_invalidating_lineage(config, lineage, original_server):
 
 
 def renew_cert(config, domains, le_client, lineage):
-    "Renew a certificate lineage."
+    # type: (interfaces.IConfig, Optional[List[str]], client.Client, storage.RenewableCert) -> None
+    """Renew a certificate lineage."""
     renewal_params = lineage.configuration["renewalparams"]
     original_server = renewal_params.get("server", cli.flag_default("server"))
     _avoid_invalidating_lineage(config, lineage, original_server)
@@ -302,11 +333,14 @@ def renew_cert(config, domains, le_client, lineage):
         domains = lineage.names()
     # The private key is the existing lineage private key if reuse_key is set.
     # Otherwise, generate a fresh private key by passing None.
-    new_key = os.path.normpath(lineage.privkey) if config.reuse_key else None
+    if config.reuse_key:
+        new_key = os.path.normpath(lineage.privkey)
+        _update_renewal_params_from_key(new_key, config)
+    else:
+        new_key = None
     new_cert, new_chain, new_key, _ = le_client.obtain_certificate(domains, new_key)
     if config.dry_run:
-        logger.debug("Dry run: skipping updating lineage at %s",
-                    os.path.dirname(lineage.cert))
+        logger.debug("Dry run: skipping updating lineage at %s", os.path.dirname(lineage.cert))
     else:
         prior_version = lineage.latest_common_version()
         # TODO: Check return value of save_successor
@@ -321,42 +355,45 @@ def report(msgs, category):
     lines = ("%s (%s)" % (m, category) for m in msgs)
     return "  " + "\n  ".join(lines)
 
+
 def _renew_describe_results(config, renew_successes, renew_failures,
                             renew_skipped, parse_failures):
+    # type: (interfaces.IConfig, List[str], List[str], List[str], List[str]) -> None
+    """
+    Print a report to the terminal about the results of the renewal process.
 
-    out = []  # type: List[str]
-    notify = out.append
-    disp = zope.component.getUtility(interfaces.IDisplay)
+    :param interfaces.IConfig config: Configuration
+    :param list renew_successes: list of fullchain paths which were renewed
+    :param list renew_failures: list of fullchain paths which failed to be renewed
+    :param list renew_skipped: list of messages to print about skipped certificates
+    :param list parse_failures: list of renewal parameter paths which had erorrs
+    """
+    notify = display_util.notify
+    notify_error = logger.error
 
-    def notify_error(err):
-        """Notify and log errors."""
-        notify(str(err))
-        logger.error(err)
+    notify('\n{}'.format(display_util.SIDE_FRAME))
+
+    renewal_noun = "simulated renewal" if config.dry_run else "renewal"
 
-    if config.dry_run:
-        notify("** DRY RUN: simulating 'certbot renew' close to cert expiry")
-        notify("**          (The test certificates below have not been saved.)")
-    notify("")
     if renew_skipped:
-        notify("The following certs are not due for renewal yet:")
+        notify("The following certificates are not due for renewal yet:")
         notify(report(renew_skipped, "skipped"))
     if not renew_successes and not renew_failures:
-        notify("No renewals were attempted.")
+        notify("No {renewal}s were attempted.".format(renewal=renewal_noun))
         if (config.pre_hook is not None or
                 config.renew_hook is not None or config.post_hook is not None):
             notify("No hooks were run.")
     elif renew_successes and not renew_failures:
-        notify("Congratulations, all renewals succeeded. The following certs "
-               "have been renewed:")
+        notify("Congratulations, all {renewal}s succeeded: ".format(renewal=renewal_noun))
         notify(report(renew_successes, "success"))
     elif renew_failures and not renew_successes:
-        notify_error("All renewal attempts failed. The following certs could "
-               "not be renewed:")
+        notify_error("All %ss failed. The following certificates could "
+               "not be renewed:", renewal_noun)
         notify_error(report(renew_failures, "failure"))
     elif renew_failures and renew_successes:
-        notify("The following certs were successfully renewed:")
+        notify("The following {renewal}s succeeded:".format(renewal=renewal_noun))
         notify(report(renew_successes, "success") + "\n")
-        notify_error("The following certs could not be renewed:")
+        notify_error("The following %ss failed:", renewal_noun)
         notify_error(report(renew_failures, "failure"))
 
     if parse_failures:
@@ -364,11 +401,7 @@ def _renew_describe_results(config, renew_successes, renew_failures,
                "were invalid: ")
         notify(report(parse_failures, "parsefail"))
 
-    if config.dry_run:
-        notify("** DRY RUN: simulating 'certbot renew' close to cert expiry")
-        notify("**          (The test certificates above have not been saved.)")
-
-    disp.notification("\n".join(out), wrap=False)
+    notify(display_util.SIDE_FRAME)
 
 
 def handle_renewal_request(config):
@@ -458,9 +491,10 @@ def handle_renewal_request(config):
 
         except Exception as e:  # pylint: disable=broad-except
             # obtain_cert (presumably) encountered an unanticipated problem.
-            logger.warning("Attempting to renew cert (%s) from %s produced an "
-                           "unexpected error: %s. Skipping.", lineagename,
-                               renewal_file, e)
+            logger.error(
+                "Failed to renew certificate %s with error: %s",
+                lineagename, e
+            )
             logger.debug("Traceback was:\n%s", traceback.format_exc())
             renew_failures.append(renewal_candidate.fullchain)
 
@@ -475,3 +509,17 @@ def handle_renewal_request(config):
     # Windows installer integration tests rely on handle_renewal_request behavior here.
     # If the text below changes, these tests will need to be updated accordingly.
     logger.debug("no renewal failures")
+
+
+def _update_renewal_params_from_key(key_path, config):
+    # type: (str, interfaces.IConfig) -> None
+    with open(key_path, 'rb') as file_h:
+        key = load_pem_private_key(file_h.read(), password=None, backend=default_backend())
+    if isinstance(key, rsa.RSAPrivateKey):
+        config.key_type = 'rsa'
+        config.rsa_key_size = key.key_size
+    elif isinstance(key, ec.EllipticCurvePrivateKey):
+        config.key_type = 'ecdsa'
+        config.elliptic_curve = key.curve.name
+    else:
+        raise errors.Error('Key at {0} is of an unsupported type: {1}.'.format(key_path, type(key)))
